/*
 * Copyright (C) 2000-2006 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch/
 *
 */
 
/**
 * \file bt_hci_defs.h
 * \author Martin Hinz <btnode@hinz.ch>
 * \date 02/08/2004
 * 
 * \author Mathias Payer <payerm@student.ethz.ch>
 * \date 2004/04/14 21:37:09

 * \author Matthias Ringwald <mringwal@inf.ethz.ch>
 *
 * @brief Defines general structs and variables
 */
 
#ifndef _BT_BT_HCI_DEFS_H_
#define _BT_BT_HCI_DEFS_H_


#include <sys/types.h>

/**
 * @brief Length of a bluetooth device address.
 */
#define BD_ADDR_LEN 6


/**
 * @brief The type bluetooth device address.
 */
typedef u_char bt_addr_t[BD_ADDR_LEN];
extern bt_addr_t bt_addr_null;


/**
 * unique app connection handle to access open baseband connections
 * ( this is different to the module connection handle internally used 
 * to communicate with the bt module )
 */
typedef u_short bt_hci_con_handle_t; /* 12-bit value*/

/**
 * this is the connection handle to communicate with the bt module
 * (not unique) and for internal use only
 */
typedef u_short _bt_hci_module_con_handle_t; /* 12-bit value*/

/**
 * indicates an invalid connection handle
 */
#define BT_HCI_HANDLE_INVALID 0xFFFF //not set to 0 since module con handle can be of value 0


/**
 * Global counter for the unique application connection handle
*/
extern bt_hci_con_handle_t bt_hci_app_con_handle_counter;


/*************************************
 * Defines for the HCI transport-layer 
 *************************************/


/**
 * Here is specified, how many open conns.
 * the bt-stack can handle.
 *
 * The number of connections the bt-controller
 * can handle is determined at startup of the stack.
*/
#define BT_HCI_MAX_NUM_CON 8

/* HCI packet types */
/**
 * Indicates (first byte) that this pkt to the bluetooth module is a command pkt. The
 * module will send a #HCI_EVENT_PACKET for response.
 */
#define HCI_COMMAND_DATA_PACKET	0x01

/**
 * Indicates (first byte) that this pkt is a acl pkt. Will be sent to and from the module.
 */
#define HCI_ACL_DATA_PACKET	    0x02

/**
 * Indicates (first byte) that this pkt is a sco pkt. Will be sent to and from the module.
 */
#define HCI_SCO_DATA_PACKET	    0x03

/**
 * Indicates (first byte) that this pkt is an event pkt. Only sent by the bluetooth module.
 */
#define HCI_EVENT_PACKET	      0x04
#define HCI_FREE_BUFFER		      0xFF
#define HCI_UNDEFINED_BUFFER	  0xFE

#define HCI_CMD_DATA_PKT_HDR	  0x03
#define HCI_ACL_DATA_PKT_HDR	  0x04
#define HCI_SCO_DATA_PKT_HDR	  0x03
#define HCI_EVENT_PKT_HDR       0x02

/** 
 * @brief The default page timeout set by the stack x 0.625 ms. 
 */
#define DEFAULT_PAGE_TIMEOUT       128*256

/** 
 * @brief The default connection accept timeout set by the stack x 0.625ms. 
 */
#define DEFAULT_CON_ACCEPT_TIMEOUT 125*256

/** 
 * @brief Get the 6-bit ogf value from a u_short address .
 * 
 * little endian (as the controller expects it):
 * we use the principle: the caller makes it right (adjusts it for the controller)
\verbatim
 0        10    16               24
 +---------+-----+----------------+--------------
 |   OCF   | OGF | PARAMETER LEN  | PARAMETER ...
 +---------+-----+----------------+--------------
\endverbatim
 * luckily our OGF values are only 6-bit wide, so we just shift ogf left by 2!
 *
 */
#define BT_HCI_GET_OGF(addr) ((*((addr)+1)>>2) & 0x3F)

/** 
 * @brief Get the 10-bit ocf value from a u_short address.
 * 
 * little endian (as the controller expects it):
 * we use the principle: the caller makes it right (adjusts it for the controller)
\verbatim
 0        10    16               24
 +---------+-----+----------------+--------------
 |   OCF   | OGF | PARAMETER LEN  | PARAMETER ...
 +---------+-----+----------------+--------------
\endverbatim
 * 
 * 
 */
#define BT_HCI_GET_OCF(addr) (((*((addr)+1)<<8) | *(addr)) & 0x3FF)


/**
 * @brief ACL Packet Boundary Flags: fragmentation continue.
 */
#define HCI_PB_CONTD_FRAGMENT  1

/**
 * @brief ACL Packet Boundary Flags: first fragmentation.
 */
#define HCI_PB_FIRST_FRAGMENT  2


/**
 * @brief ACL Broadcast Flags
 */
#define HCI_BC_UNICAST 0

/**
 * @brief ACL Broadcast Flags
 */
#define HCI_BC_ACTIVE  1

/**
 * @brief ACL Broadcast Flags
 */
#define HCI_BC_PICONET 2


// *************************************************************************
// * HCI - Commands
// * Always first the Group-Field and then the implemented cmds
// *************************************************************************

/** @brief NOP OGF for the NOP command */
#define HCI_OGF_NOP					0x00
/** @brief NOP OCF for the NOP command */
#define HCI_OCF_NOP					0x00


// *************************************************
// * LinkControl
// *************************************************
/** @brief OGF for link control commands */
#define HCI_OGF_LINK_CONTROL				   0x01

/* @brief Link-Control cmds: */
#define HCI_OCF_LC_INQUIRY				       0x01
#define HCI_OCF_LC_CREATE_CONNECTION		   0x05
#define HCI_OCF_LC_DISCONNECT				   0x06
#define HCI_OCF_LC_ACCEPT_CONNECTION_REQUEST   0x09
#define HCI_OCF_LC_REJECT_CONNECTION_REQUEST   0x0A
#define HCI_OCF_LC_LINK_KEY_REQUEST_REPLY      0x0b
#define HCI_OCF_LC_LINK_KEY_REQUEST_NEGATIVE_REPLY 0x0C
#define HCI_OCF_LC_PIN_CODE_REQUEST_REPLY      0x0d
#define HCI_OCF_LC_AUTHENTICATION_REQUESTED  0x11
#define HCI_OCF_LC_REMOTE_NAME_REQUEST		   0x19
//TODO
#define LINK_KEY_REQUEST_NEG_REPLY_OCF       0x0c
#define CHANGE_CONNECTION_PACKET_TYPE        0x0f
#define READ_REMOTE_SUPPORTED_FEATURES       0x1b
#define READ_REMOTE_VERSION_INFORMATION      0x1d


// *************************************************
// * LinkPolicy
// *************************************************
/** @brief OGF for link policy commands */
#define HCI_OGF_LINK_POLICY				0x02

/* @brief Link-Policy cmds: */ 
#define HCI_OCF_LP_HOLD_MODE                0x01
#define HCI_OCF_LP_SNIFF_MODE               0x03
#define HCI_OCF_LP_EXIT_SNIFF_MODE          0x04
#define HCI_OCF_LP_PARK_STATE               0x05
#define HCI_OCF_LP_EXIT_PARK_STATE          0x06
#define HCI_OCF_LP_ROLE_DISCOVERY			0x09
#define HCI_OCF_LP_ROLE_CHANGE				0x0B
#define HCI_OCF_LP_READ_LINK_POLICY_SETTINGS        0x0C
#define HCI_OCF_LP_WRITE_LINK_POLICY_SETTINGS		0x0D
#define HCI_OCF_LP_READ_DEFAULT_LINK_POLICY_SETTINGS    0x0E
#define HCI_OCF_LP_WRITE_DEFAULT_LINK_POLICY_SETTINGS	0x0F


// *************************************************
// * HostControl
// *************************************************
/** @brief OGF for host controller und baseband commands */
#define HCI_OGF_HOST_CONTROL				0x03

/* @brief OCF for host control commands */
#define HCI_OCF_HC_SET_EVENT_MASK                       0x01
#define HCI_OCF_HC_RESET                                0x03
#define HCI_OCF_HC_SET_EVENT_FILTER                     0x05
#define HCI_OCF_HC_READ_PIN_TYPE                        0x09
#define HCI_OCF_HC_WRITE_PIN_TYPE                       0x0A
#define HCI_OCF_HC_WRITE_LOCAL_NAME                     0x13
#define HCI_OCF_HC_READ_LOCAL_NAME                      0x14
#define HCI_OCF_HC_READ_CONNECTION_ACCEPT_TIMEOUT       0x15
#define HCI_OCF_HC_WRITE_CONNECTION_ACCEPT_TIMEOUT      0x16
#define HCI_OCF_HC_READ_PAGE_TIMEOUT                    0x17
#define HCI_OCF_HC_WRITE_PAGE_TIMEOUT                   0x18
#define HCI_OCF_HC_READ_SCAN_ENABLE                     0x19
#define HCI_OCF_HC_WRITE_SCAN_ENABLE                    0x1A
#define HCI_OCF_HC_READ_COD                             0x23
#define HCI_OCF_HC_WRITE_COD                            0x24
#define HCI_OCF_HC_SET_HOST_CTRL_TO_HOST_FLOW_CONTROL	  0x31
#define HCI_OCF_HC_HOST_BUFFER_SIZE                     0x33
#define HCI_OCF_HC_HOST_NUM_OF_COMPLETED_PACKETS_OCF	  0x35
#define HCI_OCF_HC_READ_LINK_SUPERVISION_TIMEOUT        0x36
#define HCI_OCF_HC_WRITE_LINK_SUPERVISION_TIMEOUT       0x37
#define HCI_OCF_HC_READ_INQUIRY_SCAN_TYPE               0x42
#define HCI_OCF_HC_WRITE_INQUIRY_SCAN_TYPE              0x43
#define HCI_OCF_HC_READ_INQUIRY_MODE                    0x44
#define HCI_OCF_HC_WRITE_INQUIRY_MODE                   0x45
//TODO
#define FLUSH_OCF		           0x08
#define WRITE_AUTOMATIC_FLUSH_TIMEOUT_OCF  0x28
#define READ_TRANSMIT_POWER_LEVEL          0x2d


// *************************************************
// * InformationalParameters
// *************************************************
/** @brief OGF for informational parameters */
#define HCI_OGF_INFORMATIONAL_PARAMS		0x04

/* @brief OCF for informational commands */
#define HCI_OCF_IP_READ_LOCAL_VERSION_INFORMATION  0x01
#define HCI_OCF_IP_READ_LOCAL_SUPPORTED_COMMANDS   0x02
#define HCI_OCF_IP_READ_LOCAL_SUPPORTED_FEATURES   0x03
#define HCI_OCF_IP_READ_LOCAL_EXTENDED_FEATURES    0x04
#define HCI_OCF_IP_READ_BUFFER_SIZE			       0x05
#define HCI_OCF_IP_READ_BD_ADDR				       0x09
//TODO
#define READ_COUNTRY_CODE               0x07


// *************************************************
// * StatusParameters
// *************************************************
/** @brief OGF for status parameters */
#define HCI_OGF_STATUS_PARAMS				0x05

/** @brief OCF for status commands */
#define HCI_OCF_SP_GET_LINK_QUALITY			0x03

/** @brief OCF for status commands */
#define HCI_OCF_SP_READ_RSSI				0x05


// *************************************************
// * TestCommands
// *************************************************
/** @brief OGF for test commands */
#define HCI_OGF_TEST_COMMANDS				0x06


// *************************************************
// * VendorSpecific
// *************************************************
/** @brief OGF for vendor specific commands */
#define HCI_OGF_VENDOR_COMMANDS				0x3f

/** @brief OCF for vendor specific commands */
#define HCI_OCF_VC_ERICSSON_SET_UART_BAUD_RATE	0x09

/** @brief OCF for vendor specific commands */
#define HCI_OCF_VC_ZEEVO_CHANGE_BAUD_RATE		0x0F

/** @brief OCF for vendor specific commands */
#define HCI_OCF_VC_ZEEVO_READ_ABSOLUTE_RSSI		0x12
// *************************************************************************
// * End of HCI - Commands
// *************************************************************************



// *************************************************
// * Clockset-Parameter if the value is not valid
// *************************************************
/**
 * indicates that the closck offset is valid if this bit is set
 */
#define BT_HCI_VALID_CLOCK_OFFSET	1<<15 /**< Bit 15 is set (started from 0) */


// *************************************************
// * Possible connection-types
// *************************************************
/**
 * Connection packet types DM1:\n
 * pkt size 0-17 bytes using 1 slot
 */
#define BT_HCI_PACKET_TYPE_DM1 0x0008
/**
 * Connection packet types DM1:\n
 * pkt size 0-27 bytes using 1 slot
 */
#define BT_HCI_PACKET_TYPE_DH1 0x0010
/**
 * Connection packet types DM1:\n
 * pkt size 0-121 bytes using 3 slots
 */
#define BT_HCI_PACKET_TYPE_DM3 0x0400
/**
 * Connection packet types DM1:\n
 * pkt size 0-183 bytes using 3 slots
 */
#define BT_HCI_PACKET_TYPE_DH3 0x0800
/**
 * Connection packet types DM1:\n
 * pkt size 0-224 bytes using 5 slots
 */
#define BT_HCI_PACKET_TYPE_DM5 0x4000
/**
 * Connection packet types DM1:\n
 * pkt size 0-339 bytes using 5 slots
 */
#define BT_HCI_PACKET_TYPE_DH5 0x8000


/**
 *************************************************
 * @struct bt_hci_pkt_cmd
 *************************************************
 * This is one of the possible packets types for the hci,
 * it defines a command packet from the host to the controller
*/
struct bt_hci_pkt_cmd {
	u_char type;			/**< Type of the packet */
	u_char payload[255];		/**< Payload */
};

/**
 *************************************************
 * @struct bt_hci_pkt_evt
 *************************************************
 * This is one of the possible packets types for the hci,
 * it defines an event from the controller to the host
*/
struct bt_hci_pkt_evt {
	u_char type;			/**< Type of the packet */
	u_char evt_code;		/**< Event code */
	u_char evt_length;		/**< Event length */
	u_char payload[253];		/**< Max. payload */
};


/**
 *************************************************
 * @struct bt_hci_pkt_acl
 *************************************************
 * This is one of the possible packets types for the hci,
 * it defines an acl data packet, host<->controller
*/
struct bt_hci_pkt_acl {
	u_char type;			/**< Type of the packet */
	u_char header[HCI_ACL_DATA_PKT_HDR];	/**< ACL-Header, handled by send_acl_pkt */
  /** header actually contains:
    _bt_hci_module_con_handle_t module_handle : 12;		module-handle 
	  u_short pb_flag : 2;                                Packet-Boundary flag
    u_short bc_flag : 2;                                Broadcast flag
	  u_short data_length;                                Data length
  */
	u_char payload[1];	/**< payload, size defined by user!*/
};

/**
 *************************************************
 * @struct bt_hci_pkt_sco
 *************************************************
 * This is one of the possible packets types for the hci,
 * it defines an sco data packet, host<->controller
*/
struct bt_hci_pkt_sco {
	u_char type;			/**< Type of the packet */
	u_char header[HCI_SCO_DATA_PKT_HDR];            /**SCO-Header, handled by send_acl_pkt */
  /** header contains:
	  _bt_hci_module_con_handle_t module_handle : 12;	module-handle
	  u_short reserved : 4;                           Reserved for future use
	  u_char data_length;                             Data length
  */
	u_char payload[1];	/**< payload, size defined by user*/
};

/**< Invalid / unused conn. */
#define BT_HCI_CONN_INVALID         0x00	
/**< Con. is not yet initialized (but soon will be) */
#define BT_HCI_CONN_UNINITIALIZED   0x01	
/**< Conn. is before conn. accept request */
#define BT_HCI_CONN_STATE_ACK       0x02	
/**< Create conn has been sent, waiting for ack */
#define BT_HCI_CONN_STATE_WAITACK   0x03	
/**< Accept conn has been sent, waiting for ack */
#define BT_HCI_CONN_STATE_WAITPAS   0x04	
/**< Conn. is open and we can send data */
#define BT_HCI_CONN_STATE_OPEN      0x05	

/**< Connection in active mode */
#define BT_HCI_CONN_MODE_ACTIVE     0x00  
/**< Connection in hold mode */
#define BT_HCI_CONN_MODE_HOLD       0x01    
/**< Connection in sniff mode */
#define BT_HCI_CONN_MODE_SNIFF      0x02  
/**< Connection in park state */
#define BT_HCI_CONN_MODE_PARK       0x03

/**< Connection not authenticated */
#define BT_HCI_AUTH_NONE          0x00
/**< Authentication request send */
#define BT_HCI_AUTH_SENTREQ       0x01
/**< Got PIN request  */
#define BT_HCI_AUTH_GOTPINREQ     0x02
/**< Got PIN request and sent PIN response  */
#define BT_HCI_AUTH_SENTPINRESP   0x03
/**< Connection authentication sucessfully established */
#define BT_HCI_AUTH_ESTABLISHED   0x04


/**
 *************************************************
 * @struct bt_hci_connection
 *************************************************
 * This struct defines one open connection
 * The number of possible open connections are defined above
 * Watch-out: These numbers are HW-specific!
 * This struct is for internal (stack-) use only!
*/
struct bt_hci_connection {
	bt_hci_con_handle_t app_con_handle;	/**< App handle, unique */
	_bt_hci_module_con_handle_t module_con_handle; /**< BT-Controller handle */
	bt_addr_t bdaddr;       /**< The other endpoint */
	u_short pkt_type;       /**< Pkt.type for conn (see #defines above) */
	u_long cod;             /**< The class of device */
	u_char type : 4;        /**< The connection-type (sco/acl/esco) */
	u_char role : 2;        /**< BT-Role (0: Master, 1: Slave, 3: Not sure) */
	u_char role_change : 2;	/**< Allow role-change? (0:Yes, 1: No, 3: Not sure) */
	u_char state;           /**< Connection-state */
    u_char mode;            /**< BT-mode (0: active, 1: hold, 2: sniff, 3: park) */
    u_short interval;       /**< Number of baseband slots of hold, sniff or park */
	u_char pkts_h2c;		/**< Number of uncompleted pkts, sent from the host to the controller */
    u_char auth;            /**< State of authenification (see #defines above)  */
};

/**
 * to compare two bluetooth addr
 */
#define BD_ADDR_CMP(A,B) (((A)[0]==(B)[0])&&((A)[1]==(B)[1])&&((A)[2]==(B)[2])&&((A)[3]==(B)[3])&&((A)[4]==(B)[4])&&((A)[5]==(B)[5]))
#define BD_SADDR_CMP(A,B) (((A)[0]==(B)[0])&&((A)[1]==(B)[1]))

#endif
